/* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================*/

#include <ncbi.h>
#include <dfa.h>

/*************************************************************************
 *	dfa_nextstate(S, ch, pos, report)
 *************************************************************************/
DFA_StatePtr _cdecl
dfa_nextstate(S, ch, pos, report)
	DFA_StatePtr	S;
	int	ch;
	size_t	pos;
	/* Call-back function to report matches */
	int	(_cdecl *report) PROTO((size_t pos, DFA_PatID patID));
{
	DFA_AcceptPtr	ap;
	DFA_PatlistPtr	plp;

	if (S == NULL) {
		dfaerrno = dfaErrNullParm;
		return NULL;
	}

	S = S->next[ch]; /* go to the next state upon input of the letter ch */

	if (DFA_ISACCEPTING(S)) {
		if ((ap = (DFA_AcceptPtr)DFA_UNMUNGED(S)) == NULL)
			return NULL; /* machine has halted */
		S = ap->retState;
		plp = &ap->pl;
		if (report != NULL)
			do {
				if ((*report)(pos, plp->patID) != 0) {
					dfaerrno = dfaErrReport;
					return NULL;
				}
			} while ((plp = plp->chain) != NULL);
	}
	return S;
}
